'use strict';

exports.__esModule = true;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.perspective = perspective;
exports.orthogonal = orthogonal;

var _glMatrix = require('gl-matrix');

var _object3D = require('./object3D');

var _object3D2 = _interopRequireDefault(_object3D);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function perspective(fov, near, far) {
  return function (out, aspect) {
    return _glMatrix.mat4.perspective(out, fov, aspect, near, far);
  };
}

// Zoom value may be changed often...
function orthogonal(zoom, near, far) {
  return function (out, aspect) {
    return _glMatrix.mat4.ortho(out, -aspect * zoom, aspect * zoom, -zoom, zoom, near, far);
  };
}

var Camera = function (_Object3D) {
  _inherits(Camera, _Object3D);

  function Camera() {
    var projection = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : perspective(Math.PI / 180 * 70, 0.3, 1000);

    _classCallCheck(this, Camera);

    var _this = _possibleConstructorReturn(this, _Object3D.call(this));

    _this.projection = projection;

    _this.getProjection = _this.getProjection.bind(_this);
    _this.getInverseProjection = _this.getInverseProjection.bind(_this);
    _this.getView = _this.getView.bind(_this);
    _this.getPV = _this.getPV.bind(_this);

    // This is awkward.... TODO Refactor
    _this._prevProjection = null;
    _this._aspect = 1;
    _this.projectionTicks = 0;
    _this.projectionMatrix = _glMatrix.mat4.create();

    _this.inverseProjectionTicks = 0;
    _this.inverseProjectionMatrix = _glMatrix.mat4.create();

    _this.viewTicks = 0;
    _this.viewMatrix = _glMatrix.mat4.create();

    _this._pvProjectionTicks = -1;
    _this._pvViewTicks = -1;
    _this.pvTicks = 0;
    _this.pvMatrix = _glMatrix.mat4.create();
    return _this;
  }

  Camera.prototype.get = function get(input) {
    return {
      uProjection: this.getProjection(input),
      uView: this.getView(),
      uProjectionView: this.getPV(input)
    };
  };

  Camera.prototype.getProjection = function getProjection(input) {
    var aspect = void 0;
    if (typeof input === 'number') {
      aspect = input;
    } else if ((typeof input === 'undefined' ? 'undefined' : _typeof(input)) === 'object') {
      // Received a shader object - we need to get current aspect ratio.
      aspect = input.renderer.width / input.renderer.height;
    } else {
      aspect = this._aspect;
    }
    if (this._aspect !== aspect || this._prevProjection !== this.projection) {
      this._prevProjection = this.projection;
      this._aspect = aspect;
      this.projection(this.projectionMatrix, aspect);
      this.projectionTicks++;
    }
    return this.projectionMatrix;
  };

  Camera.prototype.getInverseProjection = function getInverseProjection(input) {
    var projectionMat = this.getProjection(input);
    if (this.inverseProjectionTicks !== this.projectionTicks) {
      this.inverseProjectionTicks = this.projectionTicks;
      _glMatrix.mat4.invert(this.inverseProjectionMatrix, projectionMat);
    }
    return this.inverseProjectionMatrix;
  };

  Camera.prototype.getView = function getView() {
    var transformMat = this.transform.get();
    if (this.viewTicks !== this.transform.ticks) {
      // We can directly copy transform's ticks
      this.viewTicks = this.transform.ticks;
      _glMatrix.mat4.invert(this.viewMatrix, transformMat);
    }
    return this.viewMatrix;
  };

  Camera.prototype.getPV = function getPV(input) {
    this.getProjection(input);
    this.getView();
    if (this._pvViewTicks !== this.viewTicks || this._pvProjectionTicks !== this.projectionTicks) {
      this._pvViewTicks = this.viewTicks;
      this._pvProjectionTicks = this.projectionTicks;
      _glMatrix.mat4.multiply(this.pvMatrix, this.projectionMatrix, this.viewMatrix);
      this.pvTicks++;
    }
    return this.pvMatrix;
  };

  return Camera;
}(_object3D2.default);

exports.default = Camera;