'use strict';

exports.__esModule = true;

var _glMatrix = require('gl-matrix');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var FPSCameraController = function () {
  function FPSCameraController(node, keyNode, camera) {
    _classCallCheck(this, FPSCameraController);

    this.node = node;
    this.keyNode = keyNode;
    this.camera = camera;
    this.speed = 8.0;
    this.pitch = 0;
    this.yaw = Math.PI / 2;
    this.hasChanged = true;

    this.mouseX = 0;
    this.mouseY = 0;
    this.keys = [];

    this.registerEvents();
  }

  FPSCameraController.prototype.registerEvents = function registerEvents() {
    var _this = this;

    this.node.addEventListener('click', function () {
      _this.node.requestPointerLock = _this.node.requestPointerLock || _this.node.mozRequestPointerLock;
      _this.node.requestPointerLock();
    });
    this.node.addEventListener('mousemove', function (e) {
      if (document.pointerLockElement || document.mozPointerLockElement) {
        _this.pitch = Math.max(-Math.PI / 2 + 0.001, Math.min(Math.PI / 2 - 0.001, _this.pitch - e.movementY / 400));
        _this.yaw = _this.yaw + e.movementX / 400;
        _this.hasChanged = true;
        return;
      }
      var mouseX = e.layerX - _this.node.width / 2;
      var mouseY = e.layerY - _this.node.height / 2;
      _this.mouseX = mouseX;
      _this.mouseY = mouseY;
    });
    this.keyNode.addEventListener('keydown', function (e) {
      _this.keys[e.keyCode] = true;
    });
    this.keyNode.addEventListener('keyup', function (e) {
      _this.keys[e.keyCode] = false;
    });
  };

  FPSCameraController.prototype.update = function update(delta) {
    var camera = this.camera;


    if (!this.node.requestPointerLock && !this.node.mozRequestPointerLock) {
      if (Math.abs(this.mouseY) > 10) {
        this.pitch = Math.max(-Math.PI / 2 + 0.001, Math.min(Math.PI / 2 - 0.001, this.pitch - Math.sin(this.mouseY / 4000)));
        this.hasChanged = true;
      }
      if (Math.abs(this.mouseX) > 10) {
        this.yaw = this.yaw + Math.sin(this.mouseX / 4000);
        this.hasChanged = true;
      }
      /*this.pitch = Math.max(-Math.PI / 2 + 0.001, Math.min(Math.PI / 2
        - 0.001, -this.mouseY / 200));
      this.yaw = this.mouseX / 200;*/
    }

    // Build camera rotation.
    if (this.hasChanged) {
      _glMatrix.quat.identity(camera.transform.rotation);
      _glMatrix.quat.rotateY(camera.transform.rotation, camera.transform.rotation, -this.yaw);
      _glMatrix.quat.rotateX(camera.transform.rotation, camera.transform.rotation, this.pitch);
    }

    // Process movement; We have to build up, front, right vector.
    var up = _glMatrix.vec3.fromValues(0, 1, 0);
    // Since camera's default position faces Z-, we have to rotate them weirdly
    var front = _glMatrix.vec3.fromValues(Math.cos(this.pitch) * Math.sin(this.yaw), Math.sin(this.pitch), Math.cos(this.pitch) * -Math.cos(this.yaw));
    _glMatrix.vec3.normalize(front, front);
    // Calculating right (or left) vector can be done by calculating cross
    // vector of those two.
    var right = _glMatrix.vec3.create();
    _glMatrix.vec3.cross(right, up, front);

    var speed = this.speed * delta;
    var velocity = _glMatrix.vec3.create();
    // Now, process the key inputs.
    if (this.keys[38] || this.keys[87]) {
      _glMatrix.vec3.add(velocity, velocity, front);
    }
    if (this.keys[40] || this.keys[83]) {
      _glMatrix.vec3.subtract(velocity, velocity, front);
    }
    if (this.keys[69]) {
      _glMatrix.vec3.add(velocity, velocity, up);
    }
    if (this.keys[81]) {
      _glMatrix.vec3.subtract(velocity, velocity, up);
    }
    if (this.keys[37] || this.keys[65]) {
      _glMatrix.vec3.add(velocity, velocity, right);
    }
    if (this.keys[39] || this.keys[68]) {
      _glMatrix.vec3.subtract(velocity, velocity, right);
    }
    _glMatrix.vec3.normalize(velocity, velocity);
    _glMatrix.vec3.scale(velocity, velocity, speed);
    _glMatrix.vec3.add(camera.transform.position, camera.transform.position, velocity);
    this.hasChanged = true;
    if (this.hasChanged) {
      camera.transform.invalidate();
      this.hasChanged = false;
    }
  };

  return FPSCameraController;
}();

exports.default = FPSCameraController;