'use strict';

exports.__esModule = true;

var _quad = require('./geom/quad');

var _quad2 = _interopRequireDefault(_quad);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var VERTEX_SHADER = '\n#version 100\nattribute vec2 aPosition;\nvarying lowp vec2 vTexCoord;\n\nvoid main() {\n  gl_Position = vec4(aPosition, -1.0, 1.0);\n  vTexCoord = vec2((aPosition + 1.0) / 2.0);\n}\n';

var Filter = function () {
  function Filter(renderer, fragment, defaultOpts) {
    _classCallCheck(this, Filter);

    this.renderer = renderer;
    this.defaultOpts = defaultOpts;
    this.shader = renderer.shaders.create(VERTEX_SHADER, fragment);
    if (renderer._filterData == null) {
      renderer._filterData = {
        quad: renderer.geometries.create((0, _quad2.default)()),
        framebuffer: renderer.framebuffers.create({})
      };
    }
  }

  Filter.prototype.get = function get(input, output, options) {
    var filterData = this.renderer._filterData;
    return {
      framebuffer: {
        framebuffer: filterData.framebuffer, color: output
      },
      geometry: filterData.quad,
      shader: this.shader,
      uniforms: Object.assign({
        uTexture: input
      }, this.defaultOpts, options)
    };
  };

  return Filter;
}();

exports.default = Filter;