'use strict';

exports.__esModule = true;
exports.default = calcNormals;

var _parseAttributes = require('../util/parseAttributes');

var _parseIndices = require('../util/parseIndices');

var _parseIndices2 = _interopRequireDefault(_parseIndices);

var _glMatrix = require('gl-matrix');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function calcNormals(geometry) {
  var vertices = (0, _parseAttributes.parseAttribute)(geometry.attributes.aPosition);
  if (vertices == null) throw new Error('aPosition must be specified');
  var normals = void 0;
  if (geometry.attributes.aNormal && geometry.attributes.aNormal.data && geometry.attributes.aNormal.data.length === vertices.data.length) {
    normals = geometry.attributes.aNormal.data;
  } else {
    normals = new Float32Array(vertices.data.length);
  }
  var indices = (0, _parseIndices2.default)(geometry.indices);
  if (indices == null) throw new Error('Indices must be specified');
  // Uh, maybe the variable names are too verbose. I think.
  for (var faceId = 0; faceId < indices.length; faceId += 3) {
    var vertexId1 = indices[faceId];
    var vertexId2 = indices[faceId + 1];
    var vertexId3 = indices[faceId + 2];
    // Calculate normal vector.
    var origin = vertices.data.slice(vertexId1 * 3, vertexId1 * 3 + 3);
    var p1 = _glMatrix.vec3.create(),
        p2 = _glMatrix.vec3.create();
    var uv = _glMatrix.vec3.create();
    _glMatrix.vec3.subtract(p1, vertices.data.slice(vertexId2 * 3, vertexId2 * 3 + 3), origin);
    _glMatrix.vec3.subtract(p2, vertices.data.slice(vertexId3 * 3, vertexId3 * 3 + 3), origin);
    _glMatrix.vec3.cross(uv, p1, p2);
    _glMatrix.vec3.normalize(uv, uv);
    // Done! Paste them to those three vertices.
    normals[vertexId1 * 3] += uv[0];
    normals[vertexId1 * 3 + 1] += uv[1];
    normals[vertexId1 * 3 + 2] += uv[2];
    normals[vertexId2 * 3] += uv[0];
    normals[vertexId2 * 3 + 1] += uv[1];
    normals[vertexId2 * 3 + 2] += uv[2];
    normals[vertexId3 * 3] += uv[0];
    normals[vertexId3 * 3 + 1] += uv[1];
    normals[vertexId3 * 3 + 2] += uv[2];
  }
  // This isn't necessary for 'hard' normals, but whatever.
  for (var vertexId = 0; vertexId < normals.length; vertexId += 3) {
    var len = Math.sqrt(normals[vertexId] * normals[vertexId] + normals[vertexId + 1] * normals[vertexId + 1] + normals[vertexId + 2] * normals[vertexId + 2]);
    normals[vertexId] /= len;
    normals[vertexId + 1] /= len;
    normals[vertexId + 2] /= len;
  }
  return Object.assign({}, geometry, {
    attributes: Object.assign({}, geometry.attributes, {
      aNormal: { axis: 3, data: normals }
    })
  });
}