'use strict';

exports.__esModule = true;
exports.default = calcTangents;

var _parseAttributes = require('../util/parseAttributes');

var _parseIndices = require('../util/parseIndices');

var _parseIndices2 = _interopRequireDefault(_parseIndices);

var _glMatrix = require('gl-matrix');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function calcTangents(geometry) {
  var vertices = (0, _parseAttributes.parseAttribute)(geometry.attributes.aPosition);
  if (vertices == null) throw new Error('aPosition must be specified');
  vertices = vertices.data;
  var texCoords = (0, _parseAttributes.parseAttribute)(geometry.attributes.aTexCoord);
  if (texCoords == null) throw new Error('aTexCoord must be specified');
  texCoords = texCoords.data;
  // Resize the array
  var tangents = new Float32Array(vertices.length / 3 * 4);
  var indices = (0, _parseIndices2.default)(geometry.indices);
  if (indices == null) throw new Error('Indices must be specified');
  for (var faceId = 0; faceId < indices.length; faceId += 3) {
    var vertexId1 = indices[faceId];
    var vertexId2 = indices[faceId + 1];
    var vertexId3 = indices[faceId + 2];
    // Calculate tangent vector.
    var origin = vertices.slice(vertexId1 * 3, vertexId1 * 3 + 3);
    var p1 = _glMatrix.vec3.create();
    _glMatrix.vec3.subtract(p1, vertices.slice(vertexId2 * 3, vertexId2 * 3 + 3), origin);
    var p2 = _glMatrix.vec3.create();
    _glMatrix.vec3.subtract(p2, vertices.slice(vertexId3 * 3, vertexId3 * 3 + 3), origin);
    var texOrigin = texCoords.slice(vertexId1 * 2, vertexId1 * 2 + 2);
    var texP1 = _glMatrix.vec2.create();
    _glMatrix.vec2.subtract(texP1, texCoords.slice(vertexId2 * 2, vertexId2 * 2 + 2), texOrigin);
    var texP2 = _glMatrix.vec2.create();
    _glMatrix.vec2.subtract(texP2, texCoords.slice(vertexId3 * 2, vertexId3 * 2 + 2), texOrigin);
    // Honestly I don't know what this does.
    var f = 1 / (texP1[0] * texP2[1] - texP2[0] * texP1[1]);
    var tangent = new Float32Array(4);
    tangent[0] = f * (texP2[1] * p1[0] - texP1[1] * p2[0]);
    tangent[1] = f * (texP2[1] * p1[1] - texP1[1] * p2[1]);
    tangent[2] = f * (texP2[1] * p1[2] - texP1[1] * p2[2]);
    // Calculate bi-tangent. To save vertex array, it can be calculated in
    // vertex shader; however we have to specify the cross order to get right
    // result. This can be done by using a modifier... I think.
    // To calculate modifier, we have to calculate dot product with
    // bi-tangent from vertex shader and bi-tangent we calculated.
    var normal = _glMatrix.vec3.create();
    _glMatrix.vec3.cross(normal, p1, p2);
    _glMatrix.vec3.normalize(normal, normal);
    var leftBitangent = _glMatrix.vec3.create();
    _glMatrix.vec3.cross(leftBitangent, tangent, normal);
    // Then calculate bi-tangent with texture coords.
    var bitangent = _glMatrix.vec3.create();
    bitangent[0] = f * (texP2[0] * p1[0] - texP1[0] * p2[0]);
    bitangent[1] = f * (texP2[0] * p1[1] - texP1[0] * p2[1]);
    bitangent[2] = f * (texP2[0] * p1[2] - texP1[0] * p2[2]);
    var modifier = _glMatrix.vec3.dot(bitangent, leftBitangent);
    tangent[3] = modifier < 0 ? -1 : 1;
    // Done! Paste them to those three vertices.
    tangents.set(tangent, vertexId1 * 4);
    tangents.set(tangent, vertexId2 * 4);
    tangents.set(tangent, vertexId3 * 4);
  }
  return Object.assign({}, geometry, {
    attributes: Object.assign({}, geometry.attributes, {
      aTangent: { axis: 4, data: tangents }
    })
  });
}