'use strict';

exports.__esModule = true;
exports.default = calcNormals;

var _parseAttributes = require('../../util/parseAttributes');

var _parseIndices = require('../../util/parseIndices');

var _parseIndices2 = _interopRequireDefault(_parseIndices);

var _createIndicesArray = require('../../util/createIndicesArray');

var _createIndicesArray2 = _interopRequireDefault(_createIndicesArray);

var _glMatrix = require('gl-matrix');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function calcNormals(input) {
  var positions = (0, _parseAttributes.parseAttribute)(input.attributes.aPosition);
  var indices = (0, _parseIndices2.default)(input.indices.aPosition);
  if (positions == null || indices == null) {
    throw new Error('Position attribute and indices must be specified');
  }
  var positionsData = positions.data;
  // One vec3 per one face. so indices / 3 * 3.
  var normals = new Float32Array(indices.length);
  var normalIndices = (0, _createIndicesArray2.default)(indices.length / 3, indices.length);
  var normalId = 0;
  for (var faceId = 0; faceId < indices.length; faceId += 3) {
    var vertexId1 = indices[faceId];
    var vertexId2 = indices[faceId + 1];
    var vertexId3 = indices[faceId + 2];
    // Calculate normal vector.
    var origin = positionsData.subarray(vertexId1 * 3, vertexId1 * 3 + 3);
    var p1 = _glMatrix.vec3.create(),
        p2 = _glMatrix.vec3.create();
    var uv = _glMatrix.vec3.create();
    _glMatrix.vec3.subtract(p1, positionsData.subarray(vertexId2 * 3, vertexId2 * 3 + 3), origin);
    _glMatrix.vec3.subtract(p2, positionsData.subarray(vertexId3 * 3, vertexId3 * 3 + 3), origin);
    _glMatrix.vec3.cross(uv, p1, p2);
    _glMatrix.vec3.normalize(uv, uv);
    // Done. Store them in normals buffer, and set indices to it.
    normals.set(uv, normalId * 3);
    normalIndices[faceId] = normalId;
    normalIndices[faceId + 1] = normalId;
    normalIndices[faceId + 2] = normalId;
    normalId++;
  }
  return Object.assign({}, input, {
    attributes: Object.assign({}, input.attributes, {
      aPosition: positions, aNormal: { axis: 3, data: normals }
    }),
    indices: Object.assign({}, input.indices, {
      aPosition: indices, aNormal: normalIndices
    })
  });
}