'use strict';

exports.__esModule = true;
exports.default = calcNormals;

var _parseAttributes = require('../../util/parseAttributes');

var _parseIndices = require('../../util/parseIndices');

var _parseIndices2 = _interopRequireDefault(_parseIndices);

var _createIndicesArray = require('../../util/createIndicesArray');

var _createIndicesArray2 = _interopRequireDefault(_createIndicesArray);

var _glMatrix = require('gl-matrix');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function calcNormals(input) {
  var positions = (0, _parseAttributes.parseAttribute)(input.attributes.aPosition);
  var indices = (0, _parseIndices2.default)(input.indices.aPosition);
  if (positions == null || indices == null) {
    throw new Error('Position attribute and indices must be specified');
  }
  var positionsData = positions.data;
  // One vec3 per one vertex, since we'll normalize that later.
  var normals = new Float32Array(positionsData.length);
  for (var faceId = 0; faceId < indices.length; faceId += 3) {
    var vertexId1 = indices[faceId];
    var vertexId2 = indices[faceId + 1];
    var vertexId3 = indices[faceId + 2];
    // Calculate normal vector.
    var origin = positionsData.slice(vertexId1 * 3, vertexId1 * 3 + 3);
    var p1 = _glMatrix.vec3.create(),
        p2 = _glMatrix.vec3.create();
    var uv = _glMatrix.vec3.create();
    _glMatrix.vec3.subtract(p1, positionsData.slice(vertexId2 * 3, vertexId2 * 3 + 3), origin);
    _glMatrix.vec3.subtract(p2, positionsData.slice(vertexId3 * 3, vertexId3 * 3 + 3), origin);
    _glMatrix.vec3.cross(uv, p1, p2);
    _glMatrix.vec3.normalize(uv, uv);
    // Done. Store them in normals buffer. 'Average' buffer can be calculated
    // by adding them all, and normalizing it. So we have to add normal
    // vector to each vertex in this stage.
    normals[vertexId1 * 3] += uv[0];
    normals[vertexId1 * 3 + 1] += uv[1];
    normals[vertexId1 * 3 + 2] += uv[2];
    normals[vertexId2 * 3] += uv[0];
    normals[vertexId2 * 3 + 1] += uv[1];
    normals[vertexId2 * 3 + 2] += uv[2];
    normals[vertexId3 * 3] += uv[0];
    normals[vertexId3 * 3 + 1] += uv[1];
    normals[vertexId3 * 3 + 2] += uv[2];
  }
  // Now, normalize each normal vector.
  for (var i = 0; i < normals.length; i += 3) {
    var normal = normals.subarray(i, i + 3);
    _glMatrix.vec3.normalize(normal, normal);
  }
  // Note that we can't use tangents if we use smooth normals. (yet)
  return Object.assign({}, input, {
    attributes: Object.assign({}, input.attributes, {
      aPosition: positions, aNormal: { axis: 3, data: normals }
    }),
    indices: Object.assign({}, input.indices, {
      aPosition: indices, aNormal: indices
    })
  });
}