'use strict';

exports.__esModule = true;
exports.default = channelOld;

var _createIndicesArray = require('../../util/createIndicesArray');

var _createIndicesArray2 = _interopRequireDefault(_createIndicesArray);

var _parseAttributes = require('../../util/parseAttributes');

var _parseAttributes2 = _interopRequireDefault(_parseAttributes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function channelOld(input) {
  var attributes = (0, _parseAttributes2.default)(input.attributes);
  var indices = input.indices;
  var indicesSize = -1;
  var vertexCount = -1;
  var outputAttribs = {};
  var outputIndices = void 0;
  // First, verify the indices size and populate the attrib arrays.
  for (var key in indices) {
    // The attribute must be present
    if (attributes[key] == null) {
      throw new Error('Attribute ' + key + ' is not defined');
    }
    var indicesObj = indices[key];
    if (indicesObj == null) continue;
    // Save indices size if not initialized yet.
    if (indicesSize === -1) {
      indicesSize = indicesObj.length;
      // Since it doesn't do any optimization yet, we can assume that vertex
      // count and indices size are same.
      vertexCount = indicesObj.length;
    }
    // Verify the indices size.
    if (indicesObj.length !== indicesSize) {
      throw new Error('Indices size does not match');
    }
    // Populate the attribute array.
    // TODO Support more than Float32Array?
    var axis = attributes[key].axis;
    outputAttribs[key] = {
      axis: axis, data: new Float32Array(vertexCount * axis)
    };
  }
  // Then, populate the indices array.
  outputIndices = (0, _createIndicesArray2.default)(vertexCount, indicesSize);
  // Iterate through each indices and put the attribute data.
  for (var _key in indices) {
    var attribute = outputAttribs[_key];
    if (attribute == null) continue;
    var buffer = attribute.data;
    var _axis = attribute.axis;
    var originalBuffer = attributes[_key].data;
    var indicesBuffer = indices[_key];
    for (var i = 0; i < indicesSize; ++i) {
      var originalPos = indicesBuffer[i];
      var original = originalBuffer.slice(_axis * originalPos, _axis * originalPos + _axis);
      buffer.set(original, _axis * i);
    }
  }
  // Put the indices data.
  for (var _i = 0; _i < indicesSize; ++_i) {
    outputIndices[_i] = _i;
  }
  return Object.assign({}, input, {
    attributes: outputAttribs,
    indices: outputIndices
  });
}