'use strict';

exports.__esModule = true;
exports.calcNormals = exports.calcSmoothNormals = exports.calcTangents = exports.channelOld = undefined;

var _channelOld = require('./channelOld');

Object.defineProperty(exports, 'channelOld', {
  enumerable: true,
  get: function get() {
    return _interopRequireDefault(_channelOld).default;
  }
});

var _calcTangents = require('./calcTangents');

Object.defineProperty(exports, 'calcTangents', {
  enumerable: true,
  get: function get() {
    return _interopRequireDefault(_calcTangents).default;
  }
});

var _calcSmoothNormals = require('./calcSmoothNormals');

Object.defineProperty(exports, 'calcSmoothNormals', {
  enumerable: true,
  get: function get() {
    return _interopRequireDefault(_calcSmoothNormals).default;
  }
});

var _calcNormals = require('./calcNormals');

Object.defineProperty(exports, 'calcNormals', {
  enumerable: true,
  get: function get() {
    return _interopRequireDefault(_calcNormals).default;
  }
});
exports.default = channel;

var _parseAttributes = require('../../util/parseAttributes');

var _parseAttributes2 = _interopRequireDefault(_parseAttributes);

var _parseIndices = require('../../util/parseIndices');

var _parseIndices2 = _interopRequireDefault(_parseIndices);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function channel(input) {
  if (Array.isArray(input)) return input.map(function (v) {
    return channel(v);
  });
  var attributes = (0, _parseAttributes2.default)(input.attributes);
  var indicesArr = [];
  var indicesSize = -1;

  var indicesCache = {};
  var vertexCount = 0;

  var outputAttribs = {};
  var outputIndices = [];
  // First, verify the indices size and populate the attrib arrays.
  for (var key in input.indices) {
    var attribObj = attributes[key];
    // The attribute must be present
    if (attribObj == null) {
      throw new Error('Attribute ' + key + ' is not defined');
    }
    var indicesObj = (0, _parseIndices2.default)(input.indices[key]);
    if (indicesObj == null) continue;
    outputAttribs[key] = [];
    indicesArr.push({
      indices: indicesObj,
      data: attribObj.data,
      axis: attribObj.axis,
      outData: outputAttribs[key]
    });
    // Save indices size if not initialized yet.
    if (indicesSize === -1) {
      indicesSize = indicesObj.length;
    }
    // Verify the indices size.
    if (indicesObj.length !== indicesSize) {
      throw new Error('Indices size does not match');
    }
  }
  // Populate indices array while generating attributes data.

  var _loop = function _loop(i) {
    // Validate cache
    var key = indicesArr.map(function (_ref) {
      var indices = _ref.indices;
      return indices[i];
    }).join('/');
    var index = indicesCache[key];
    if (index == null) {
      indicesArr.forEach(function (_ref2) {
        var indices = _ref2.indices,
            axis = _ref2.axis,
            data = _ref2.data,
            outData = _ref2.outData;

        var offset = indices[i] * axis;
        if (offset + axis > data.length) {
          outData.push(new Float32Array(axis));
        } else {
          outData.push(data.subarray(offset, offset + axis));
        }
      });
      index = indicesCache[key] = vertexCount;
      vertexCount++;
    }
    outputIndices.push(index);
  };

  for (var i = 0; i < indicesSize; ++i) {
    _loop(i);
  }
  return Object.assign({}, input, {
    attributes: (0, _parseAttributes2.default)(outputAttribs),
    indices: (0, _parseIndices2.default)(outputIndices)
  });
}