'use strict';

exports.__esModule = true;
exports.default = grid;

var _geometry = require('../renderer/geometry');

// Generates 2D grid geometry
function grid() {
  var width = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 17;
  var height = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 17;
  var gap = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;

  var vertices = new Float32Array(4 * (width + height));
  var minX = -(width - 1) / 2 * gap;
  var minY = -(height - 1) / 2 * gap;
  var maxX = (width - 1) / 2 * gap;
  var maxY = (height - 1) / 2 * gap;
  var pos = 0;
  for (var y = 0; y < height; ++y) {
    var offsetY = (y - (height - 1) / 2) * gap;
    vertices[pos++] = minX;
    vertices[pos++] = offsetY;
    vertices[pos++] = maxX;
    vertices[pos++] = offsetY;
  }
  for (var x = 0; x < width; ++x) {
    var offsetX = (x - (width - 1) / 2) * gap;
    vertices[pos++] = offsetX;
    vertices[pos++] = minY;
    vertices[pos++] = offsetX;
    vertices[pos++] = maxY;
  }
  return {
    attributes: {
      aPosition: { axis: 2, data: vertices }
    },
    mode: _geometry.LINES
  };
}