"use strict";

exports.__esModule = true;
exports.default = quad;
// Uni-directional quad geometry used by post-processing scene.
function quad() {
  var hSlice = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
  var vSlice = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;

  var vertices = new Float32Array((hSlice + 1) * (vSlice + 1) * 3);
  var texCoords = new Float32Array((hSlice + 1) * (vSlice + 1) * 2);
  var indices = void 0;
  if ((hSlice + 1) * (vSlice + 1) > 65535) {
    indices = new Uint32Array(hSlice * vSlice * 6);
  } else {
    indices = new Uint16Array(hSlice * vSlice * 6);
  }
  // Mark vertices
  for (var y = 0; y <= vSlice; ++y) {
    var yPos = y / vSlice * 2 - 1;
    for (var x = 0; x <= hSlice; ++x) {
      var xPos = x / hSlice * 2 - 1;
      var pos = y * (hSlice + 1) + x;
      vertices[pos * 3] = xPos;
      vertices[pos * 3 + 1] = yPos;
      vertices[pos * 3 + 2] = 0;
      texCoords[pos * 2] = (xPos + 1) / 2;
      texCoords[pos * 2 + 1] = (yPos + 1) / 2;
    }
  }
  // Mark indices
  for (var _y = 0; _y < vSlice; ++_y) {
    for (var _x3 = 0; _x3 < hSlice; ++_x3) {
      // Vertex indices
      var tl = _y * (hSlice + 1) + _x3;
      var tr = _y * (hSlice + 1) + _x3 + 1;
      var bl = (_y + 1) * (hSlice + 1) + _x3;
      var br = (_y + 1) * (hSlice + 1) + _x3 + 1;
      // Actual index position
      var _pos = (_y * hSlice + _x3) * 6;
      indices[_pos] = tl;
      indices[_pos + 1] = tr;
      indices[_pos + 2] = br;
      indices[_pos + 3] = br;
      indices[_pos + 4] = bl;
      indices[_pos + 5] = tl;
    }
  }
  return {
    attributes: {
      aPosition: { axis: 3, data: vertices },
      aTexCoord: { axis: 2, data: texCoords }
    }, indices: indices
  };
}