'use strict';

exports.__esModule = true;
exports.default = unwrapInstanced;

var _parseAttributes = require('../util/parseAttributes');

var _parseAttributes2 = _interopRequireDefault(_parseAttributes);

var _parseIndices = require('../util/parseIndices');

var _parseIndices2 = _interopRequireDefault(_parseIndices);

var _createIndicesArray = require('../util/createIndicesArray');

var _createIndicesArray2 = _interopRequireDefault(_createIndicesArray);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// import { LINE_LOOP, LINE_STRIP, TRIANGLE_STRIP, TRIANGLE_FAN } from
//  '../renderer/geometry';

function unwrapInstanced(input) {
  var instanced = input.instanced;
  var attributes = (0, _parseAttributes2.default)(input.attributes);
  var indices = (0, _parseIndices2.default)(input.indices);
  if (instanced == null) return input;
  var outAttributes = {};
  var outIndices = null;
  // TODO Check mode
  // Check vertex count and prim count
  var vertexCount = -1;
  var primCount = -1;
  for (var key in attributes) {
    var attribute = attributes[key];
    var instDiv = instanced[key] || 0;
    if (instDiv !== 0) {
      if (primCount === -1) {
        primCount = attribute.data.length / attribute.axis * instDiv;
      } else if (attribute.data.length !== primCount * attribute.axis / instDiv) {
        throw new Error('Instanced primitive data size mismatch');
      }
    } else {
      if (vertexCount === -1) {
        vertexCount = attribute.data.length / attribute.axis;
      } else if (vertexCount !== attribute.data.length / attribute.axis) {
        throw new Error('Vertex count mismatch');
      }
    }
  }
  if (primCount <= 1) {
    // Return input with 'instanced' value removed
    return Object.assign({}, input, { instanced: undefined });
  }
  // Populate attributes
  for (var _key in attributes) {
    var _attribute = attributes[_key];
    var _instDiv = 0;
    if (instanced != null) _instDiv = instanced[_key] || 0;
    if (_instDiv !== 0) {
      // Instanced attribute
      // Support other than Float32Array
      var out = new Float32Array(vertexCount * _attribute.axis * primCount);
      // Copy the data
      for (var i = 0; i < primCount; ++i) {
        var outOffset = vertexCount * _attribute.axis * i;
        var offset = i / _instDiv | 0;
        var primData = _attribute.data.subarray(offset * _attribute.axis, (offset + 1) * _attribute.axis);
        for (var j = 0; j < vertexCount; ++j) {
          out.set(primData, outOffset + j * _attribute.axis);
        }
      }
      outAttributes[_key] = {
        axis: _attribute.axis, data: out
      };
    } else {
      // Support other than Float32Array
      var _out = new Float32Array(vertexCount * _attribute.axis * primCount);
      // Copy the data....
      for (var _i = 0; _i < primCount; ++_i) {
        _out.set(_attribute.data, vertexCount * _attribute.axis * _i);
      }
      outAttributes[_key] = {
        axis: _attribute.axis, data: _out
      };
    }
  }
  // Populate indices
  if (indices) {
    outIndices = (0, _createIndicesArray2.default)(vertexCount * primCount, indices.length * primCount);
    for (var _i2 = 0; _i2 < primCount; ++_i2) {
      var _offset = _i2 * indices.length;
      // Since we have to increment value, we can't just copy it
      for (var _j = 0; _j < indices.length; ++_j) {
        outIndices[_offset + _j] = indices[_j] + vertexCount * _i2;
      }
    }
  }
  // All done! export the value
  return {
    attributes: outAttributes,
    indices: outIndices,
    mode: input.mode
  };
}