'use strict';

exports.__esModule = true;
exports.default = wireframe;

var _geometry = require('../renderer/geometry');

var _getVerticesCount = require('../util/getVerticesCount');

var _getVerticesCount2 = _interopRequireDefault(_getVerticesCount);

var _createIndicesArray = require('../util/createIndicesArray');

var _createIndicesArray2 = _interopRequireDefault(_createIndicesArray);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function wireframe(input) {
  if (Array.isArray(input)) return input.map(function (v) {
    return wireframe(v);
  });
  if (input.mode != null && input.mode != _geometry.TRIANGLES) return input;
  var verticesCount = (0, _getVerticesCount2.default)(input.attributes);
  if (input.indices) {
    var indices = (0, _createIndicesArray2.default)(verticesCount, input.indices.length * 2);
    // Rebuild indices
    for (var i = 0; i < Math.floor(input.indices.length / 3); ++i) {
      indices[i * 6] = input.indices[i * 3];
      indices[i * 6 + 1] = input.indices[i * 3 + 1];
      indices[i * 6 + 2] = input.indices[i * 3 + 1];
      indices[i * 6 + 3] = input.indices[i * 3 + 2];
      indices[i * 6 + 4] = input.indices[i * 3 + 2];
      indices[i * 6 + 5] = input.indices[i * 3];
    }
    return Object.assign({}, input, { indices: indices, mode: _geometry.LINES });
  } else {
    // Linear mode; generate indices using vertices count
    var _indices = (0, _createIndicesArray2.default)(verticesCount, verticesCount * 2);
    // Rebuild indices
    for (var _i = 0; _i < Math.floor(verticesCount / 3); ++_i) {
      _indices[_i * 6] = _i * 3;
      _indices[_i * 6 + 1] = _i * 3 + 1;
      _indices[_i * 6 + 2] = _i * 3 + 1;
      _indices[_i * 6 + 3] = _i * 3 + 2;
      _indices[_i * 6 + 4] = _i * 3 + 2;
      _indices[_i * 6 + 5] = _i * 3;
    }
    return Object.assign({}, input, { indices: _indices, mode: _geometry.LINES });
  }
}