"use strict";

exports.__esModule = true;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Buffer = function () {
  function Buffer(renderer, data, usage) {
    _classCallCheck(this, Buffer);

    this.renderer = renderer;
    this.data = data;
    if (Array.isArray(data)) {
      this.data = new Float32Array(data);
    }
    this.usage = usage == null ? renderer.gl.STATIC_DRAW : usage;
    this.vbo = null;
  }

  Buffer.prototype.update = function update(data) {
    this.data = data;
    if (Array.isArray(data)) {
      this.data = new Float32Array(data);
    }
    this.upload();
  };

  Buffer.prototype.updateSub = function updateSub(data, offset) {
    if (this.vbo == null) this.upload();
    // Update subdata...
    this.data.set(data, offset);
    var gl = this.renderer.gl;
    gl.bindBuffer(gl.ARRAY_BUFFER, this.vbo);
    gl.bufferSubData(gl.ARRAY_BUFFER, offset * 4, data);
    gl.bindBuffer(gl.ARRAY_BUFFER, null);
  };

  Buffer.prototype.upload = function upload() {
    var gl = this.renderer.gl;
    if (this.vbo == null) this.vbo = gl.createBuffer();
    // Upload whole data... Nothing special
    gl.bindBuffer(gl.ARRAY_BUFFER, this.vbo);
    gl.bufferData(gl.ARRAY_BUFFER, this.data, this.usage);
    gl.bindBuffer(gl.ARRAY_BUFFER, null);
  };

  Buffer.prototype.dispose = function dispose() {
    var gl = this.renderer.gl;
    if (this.buffer == null) return;
    gl.deleteBuffer(this.vbo);
    this.buffer = null;
  };

  return Buffer;
}();

exports.default = Buffer;