'use strict';

exports.__esModule = true;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// Why don't we use an object? We don't know the constant value..
function getAttachment(gl, format) {
  // Doesn't support extension for now.
  switch (format) {
    case gl.RGBA4:
    case gl.RGB565:
    case gl.RGR5_A1:
      return gl.COLOR_ATTACHMENT0;
    case gl.DEPTH_COMPONENT16:
      return gl.DEPTH_ATTACHMENT;
    case gl.STENCIL_INDEX8:
      return gl.STENCIL_ATTACHMENT;
    case gl.DEPTH_STENCIL:
      return gl.DEPTH_STENCIL_ATTACHMENT;
  }
}

var Framebuffer = function () {
  function Framebuffer(renderer, options) {
    _classCallCheck(this, Framebuffer);

    this.renderer = renderer;
    this.options = options;

    this.framebuffer = null;
    // Renderbuffers
    this.color = null;
    this.depth = null;
    // this.stencil = null;
  }

  Framebuffer.prototype.init = function init(input) {
    var gl = this.renderer.gl;
    this.framebuffer = gl.createFramebuffer();
    gl.bindFramebuffer(gl.FRAMEBUFFER, this.framebuffer);
    var options = input || this.options;
    // Upload the textures and renderbuffers
    // Determine the size of the framebuffer
    var width = gl.drawingBufferWidth;
    var height = gl.drawingBufferHeight;
    // TODO Color only supports texture attachment
    // Also, no stencil for now, and no draw_buffers extension.
    if (options.color && typeof options.color !== 'number') {
      var texture = options.color;
      var target = gl.TEXTURE_2D;
      if (options.color.renderer == null) {
        texture = options.color.texture;
        target = options.color.target;
      }
      if (!texture.loaded || texture.varyingSize && !texture.validateSize()) {
        this.renderer.textures.use(0, texture, true);
      }
      gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, target, texture.texture, 0);
      width = texture.width;
      height = texture.height;
      this.color = texture;
    } else {
      // ???
      throw new Error('Color attachment must be specified');
    }
    // Bind depth... should support textures too.
    if (this.options.depth && typeof this.options.depth === 'number') {
      // Create renderbuffer if it doesn't exists.
      if (this.depth == null) {
        this.depth = gl.createRenderbuffer();
      }
      gl.bindRenderbuffer(gl.RENDERBUFFER, this.depth);
      // TODO Should reset renderbuffer if it has different size.
      gl.renderbufferStorage(gl.RENDERBUFFER, this.options.depth, width, height);
      // Bind the renderbuffer.
      gl.framebufferRenderbuffer(gl.FRAMEBUFFER, getAttachment(gl, this.options.depth), gl.RENDERBUFFER, this.depth);
    }
    // Good enough! we're done. Kind of.
    this.width = width;
    this.height = height;
  };

  Framebuffer.prototype.rebind = function rebind(options) {
    var gl = this.renderer.gl;
    gl.bindFramebuffer(gl.FRAMEBUFFER, this.framebuffer);
    if (options.color && typeof options.color !== 'number') {
      this.options.color = options.color;
      var texture = options.color;
      var target = gl.TEXTURE_2D;
      if (options.color.renderer == null) {
        texture = options.color.texture;
        target = options.color.target;
      }
      if (!texture.loaded || texture.varyingSize && !texture.validateSize()) {
        this.renderer.textures.use(0, texture, true);
      }
      gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, target, texture.texture, 0);
      this.width = texture.width;
      this.height = texture.height;
      this.color = texture;
    }
  };

  Framebuffer.prototype.resize = function resize() {
    var gl = this.renderer.gl;
    gl.bindFramebuffer(gl.FRAMEBUFFER, this.framebuffer);
    var options = this.options;
    // Upload the textures and renderbuffers
    // Determine the size of the framebuffer
    var width = gl.drawingBufferWidth;
    var height = gl.drawingBufferHeight;
    // TODO Color only supports texture attachment
    // Also, no stencil for now, and no draw_buffers extension.
    if (options.color && typeof options.color !== 'number') {
      var texture = options.color;
      var target = gl.TEXTURE_2D;
      if (options.color.renderer == null) {
        texture = options.color.texture;
        target = options.color.target;
      }
      if (!texture.loaded || texture.varyingSize && !texture.validateSize()) {
        this.renderer.textures.use(0, texture, true);
      }
      gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, target, texture.texture, 0);
      width = texture.width;
      height = texture.height;
      this.color = texture;
    } else {
      // ???
      throw new Error('Color attachment must be specified');
    }
    // Bind depth... should support textures too.
    if (options.depth && typeof options.depth === 'number') {
      if (this.width !== width || this.height !== height) {
        gl.bindRenderbuffer(gl.RENDERBUFFER, this.depth);
        // TODO Should reset renderbuffer if it has different size.
        gl.renderbufferStorage(gl.RENDERBUFFER, options.depth, width, height);
        // Bind the renderbuffer.
        gl.framebufferRenderbuffer(gl.FRAMEBUFFER, getAttachment(gl, options.depth), gl.RENDERBUFFER, this.depth);
      }
    }
    // Good enough! we're done. Kind of.
    this.width = width;
    this.height = height;
  };

  Framebuffer.prototype.validateSize = function validateSize() {
    if (this.color != null && this.color.varyingSize && !this.color.validateSize()) {
      return false;
    }
    return this.color.width === this.width && this.color.height === this.height;
  };

  Framebuffer.prototype.readPixels = function readPixels(x, y, width, height, format, type, pixels) {
    var gl = this.renderer.gl;
    // Use itself
    this.renderer.framebuffers.use(this);
    return gl.readPixels(x, y, width, height, format, type, pixels);
  };

  Framebuffer.prototype.readPixelsRGBA = function readPixelsRGBA(x, y, width, height, pixels) {
    var gl = this.renderer.gl;
    if (pixels instanceof Uint8Array) {
      return this.readPixels(x, y, width, height, gl.RGBA, gl.UNSIGNED_BYTE, pixels);
    } else if (pixels instanceof Float32Array) {
      return this.readPixels(x, y, width, height, gl.RGBA, gl.FLOAT, pixels);
    } else {
      throw new Error('Uint16Array is not supported (use readPixels)');
    }
  };

  Framebuffer.prototype.readPixelsRGB = function readPixelsRGB(x, y, width, height, pixels) {
    var gl = this.renderer.gl;
    if (pixels instanceof Uint8Array) {
      return this.readPixels(x, y, width, height, gl.RGB, gl.UNSIGNED_BYTE, pixels);
    } else if (pixels instanceof Float32Array) {
      return this.readPixels(x, y, width, height, gl.RGB, gl.FLOAT, pixels);
    } else {
      throw new Error('Uint16Array is not supported (use readPixels)');
    }
  };

  Framebuffer.prototype.use = function use(options) {
    if (this.framebuffer == null) return this.init(options);
    if (options != null) return this.rebind(options);
    if (!this.validateSize()) {
      return this.resize();
    }
    var gl = this.renderer.gl;
    // Too easy
    gl.bindFramebuffer(gl.FRAMEBUFFER, this.framebuffer);
  };

  Framebuffer.prototype.dispose = function dispose() {
    var gl = this.renderer.gl;
    if (this.framebuffer == null) return;
    gl.deleteFramebuffer(this.framebuffer);
    this.framebuffer = null;
  };

  return Framebuffer;
}();

exports.default = Framebuffer;