'use strict';

exports.__esModule = true;

var _framebuffer = require('./framebuffer');

var _framebuffer2 = _interopRequireDefault(_framebuffer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var FramebufferManager = function () {
  function FramebufferManager(renderer) {
    _classCallCheck(this, FramebufferManager);

    this.renderer = renderer;
    this.framebuffers = [];
    this.current = null;
  }

  FramebufferManager.prototype.create = function create(options) {
    var framebuffer = new _framebuffer2.default(this.renderer, options);
    this.framebuffers.push(framebuffer);
    return framebuffer;
  };

  FramebufferManager.prototype.use = function use(input) {
    var framebuffer = input;
    var options = void 0;
    if (framebuffer == null || framebuffer === false) {
      if (this.current == null) return;
      var gl = this.renderer.gl;
      gl.bindFramebuffer(gl.FRAMEBUFFER, null);
      this.current = null;
      return;
    }
    if (input.renderer == null) {
      options = input;
      framebuffer = input.framebuffer;
    }
    if (this.current === framebuffer && options == null) return;
    framebuffer.use(options);
    this.current = framebuffer;
  };

  FramebufferManager.prototype.reset = function reset() {
    this.current = null;
    this.framebuffers.forEach(function (framebuffer) {
      return framebuffer.dispose();
    });
  };

  return FramebufferManager;
}();

exports.default = FramebufferManager;