'use strict';

exports.__esModule = true;

var _geometry = require('./geometry');

var _geometry2 = _interopRequireDefault(_geometry);

var _buffer = require('./buffer');

var _buffer2 = _interopRequireDefault(_buffer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var GeometryManager = function () {
  function GeometryManager(renderer) {
    _classCallCheck(this, GeometryManager);

    this.renderer = renderer;
    this.geometries = [];
    this.buffers = [];
    this.current = null;
  }

  GeometryManager.prototype.createBuffer = function createBuffer(data) {
    var buffer = new _buffer2.default(this.renderer, data);
    this.buffers.push(buffer);
    return buffer;
  };

  GeometryManager.prototype.create = function create(options) {
    var _this = this;

    if (Array.isArray(options)) {
      return options.map(function (v) {
        return _this.create(v);
      });
    }
    var finalOpts = options;
    // Old geometry compatibility code
    if (options.getAttributes) {
      finalOpts = {};
      finalOpts.attributes = options.getAttributes();
      finalOpts.indices = options.getIndices();
      switch (options.type) {
        case 'points':
          finalOpts.mode = this.renderer.gl.POINTS;
          break;
        case 'lines':
          finalOpts.mode = this.renderer.gl.LINES;
          break;
        case 'triangles':
        default:
          finalOpts.mode = this.renderer.gl.TRIANGLES;
          break;
      }
    }
    // attributes, indices, mode (or passes)
    var geometry = new _geometry2.default(this.renderer, finalOpts);
    this.geometries.push(geometry);
    return geometry;
  };

  GeometryManager.prototype.use = function use(geometry) {
    if (Array.isArray(geometry)) {
      // Can't use :/
      this.current = geometry;
      return;
    }
    if (geometry == null) return;
    geometry.use();
    this.current = geometry;
  };

  GeometryManager.prototype.draw = function draw() {
    if (Array.isArray(this.current)) {
      this.current.forEach(function (v) {
        v.use();
        v.draw();
      });
      return;
    }
    this.current.draw();
  };

  GeometryManager.prototype.reset = function reset() {
    this.current = null;
    this.geometries.forEach(function (geometry) {
      return geometry.dispose();
    });
    this.buffers.forEach(function (buffer) {
      return buffer.dispose();
    });
  };

  return GeometryManager;
}();

exports.default = GeometryManager;