'use strict';

exports.__esModule = true;

var _preprocessShader = require('./preprocessShader');

var _preprocessShader2 = _interopRequireDefault(_preprocessShader);

var _shader = require('./shader');

var _shader2 = _interopRequireDefault(_shader);

var _traverseNode = require('../util/traverseNode');

var _traverseNode2 = _interopRequireDefault(_traverseNode);

var _checkFrustum = require('../util/checkFrustum');

var _checkFrustum2 = _interopRequireDefault(_checkFrustum);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ShaderManager = function () {
  function ShaderManager(renderer) {
    _classCallCheck(this, ShaderManager);

    this.renderer = renderer;
    this.shaders = [];
    this.current = null;
    this.handler = null;
    // Preprocess shader's governor list
    this.governors = {
      max: {
        checker: function checker(shader, current) {
          return shader >= current;
        },
        allocator: function allocator(current) {
          return current;
        }
      },
      equal: {
        checker: function checker(shader, current) {
          return shader === current;
        },
        allocator: function allocator(current) {
          return current;
        }
      },
      maxLength: {
        checker: function checker(shader, current) {
          return shader >= (current == null ? 0 : current.length);
        },
        allocator: function allocator(current) {
          return current == null ? 0 : current.length;
        }
      }
    };
    this.checkFrustum = _checkFrustum2.default;
  }

  ShaderManager.prototype.create = function create(vert, frag) {
    var noPreprocess = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

    var shader = void 0;
    if (noPreprocess) {
      shader = new _shader2.default(this.renderer, vert, frag);
    } else {
      shader = new _preprocessShader2.default(this.renderer, vert, frag);
    }
    this.shaders.push(shader);
    return shader;
  };

  ShaderManager.prototype.useNode = function useNode(node) {
    var shader = node.shader;
    if (shader == null || shader === false) return false;
    // First, we pre-calculate uniforms before doing anything else.
    // However, this is only for 'pre-process' shader, so we skip if
    // invaliators are not defined.
    if (shader.invalidators && (shader.useCounts || shader.useFeatures)) {
      (0, _traverseNode2.default)(shader.currentNode, node, function (v) {
        shader.ascendNode(v);
      }, function (v) {
        shader.descendNode(v);
      }, function () {
        shader.currentUniforms = {};
      });
      shader.currentNode = node;
      shader = shader.getShader();
    }
    if (this.handler != null) {
      shader = this.handler(shader, node, this.renderer);
    }
    if (shader == null || shader === false) return false;
    shader = shader.use(undefined, this.current);
    this.current = shader;
    // Traverse to the uniforms
    (0, _traverseNode2.default)(shader.currentNode, node, function () {}, function (v) {
      if (v.data.uniforms != null) shader.setUniforms(v.data.uniforms);
    });
    shader.currentNode = node;
  };

  ShaderManager.prototype.use = function use(shader, uniforms) {
    var returned = shader.getShader(uniforms, this.current);
    if (this.handler != null) {
      returned = this.handler(returned, uniforms, this.renderer);
    }
    this.current = returned.use(uniforms, this.current);
  };

  ShaderManager.prototype.reset = function reset() {
    this.current = null;
    this.shaders.forEach(function (shader) {
      return shader.dispose();
    });
  };

  return ShaderManager;
}();

exports.default = ShaderManager;