'use strict';

exports.__esModule = true;

var _parseUniform = require('../util/parseUniform');

var _parseUniform2 = _interopRequireDefault(_parseUniform);

var _traverseNode = require('../util/traverseNode');

var _traverseNode2 = _interopRequireDefault(_traverseNode);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var BIT_POS = {
  blend: 1,
  cull: 2,
  depth: 4,
  dither: 8,
  polygonOffset: 16,
  scissor: 32,
  stencil: 64,
  colorMask: 128,
  depthMask: 256
};

var StateManager = function () {
  function StateManager(renderer) {
    _classCallCheck(this, StateManager);

    this.doReset = false;
    this.renderer = renderer;
    this.state = 0;
    this.currentNode = null;
    /*this.state = {
      blend: {
        enabled: false,
        color: new Float32Array([1, 1, 1, 1]),
        equation: {
          rgb: renderer.gl.FUNC_ADD,
          alpha: renderer.gl.FUNC_ADD
        },
        func: {
          rgb: [renderer.gl.ONE, renderer.gl.ZERO],
          alpha: [renderer.gl.ONE, renderer.gl.ZERO]
        }
      },
      clearColor: new Float32Array([0, 0, 0, 0]),
      clearDepth: 1,
      clearStencil: 0,
      colorMask: [true, true, true, true],
      // TODO merge into one?
      cull: {
        enabled: false,
        face: renderer.gl.FRONT,
        front: renderer.gl.CW
      },
      depth: {
        enabled: false,
        func: renderer.gl.LESS,
        mask: true,
        range: new Float32Array(0, 1)
      },
      dither: false,
      polygonOffset: {
        enabled: false,
        factor: 0,
        units: 0
      },
      scissor: false,
      stencil: {
        enabled: false,
        front: {
          func: {
            func: renderer.gl.ALWAYS,
            ref: 0,
            mask: ~0
          },
          mask: ~0,
          op: {
            fail: renderer.gl.KEEP,
            zfail: renderer.gl.KEEP,
            zpass: renderer.gl.KEEP
          }
        },
        back: {
          func: {
            func: renderer.gl.ALWAYS,
            ref: 0,
            mask: ~0
          },
          mask: ~0,
          op: {
            fail: renderer.gl.KEEP,
            zfail: renderer.gl.KEEP,
            zpass: renderer.gl.KEEP
          }
        }
      }
    };*/
  }

  StateManager.prototype.reset = function reset(options) {
    // Resets the state (Basically, disables everything unless specified.)
    this.set(options || {}, true);
  };

  StateManager.prototype.setEnabled = function setEnabled(key, pos, value) {
    var gl = this.renderer.gl;
    if (value == null && !this.doReset) return;
    // If value is not 'false', it's enabled.
    var enabled = value !== false && value != null;
    if ((this.state & pos) !== 0 === enabled) return;
    // Set the bit according to the bit set.
    if (enabled) {
      this.state |= pos;
      gl.enable(key);
    } else {
      this.state &= ~pos;
      gl.disable(key);
    }
  };

  StateManager.prototype.setBlend = function setBlend(options) {
    var gl = this.renderer.gl;
    this.setEnabled(gl.BLEND, BIT_POS.blend, options);
    if (!options) return;
    // Will it be safe?
    if (options.color) {
      var arr = options.color;
      gl.blendColor(arr[0], arr[1], arr[2], arr[3]);
    }
    if (options.equation != null) {
      if (typeof options.equation === 'number') {
        gl.blendEquation(options.equation);
      } else {
        gl.blendEquationSeparate(options.equation.rgb, options.equation.alpha);
      }
    }
    if (options.func) {
      if (Array.isArray(options.func)) {
        gl.blendFunc(options.func[0], options.func[1]);
      } else {
        gl.blendFuncSeparate(options.func.rgb[0], options.func.rgb[1], options.func.alpha[0], options.func.alpha[1]);
      }
    }
  };

  StateManager.prototype.setColorMask = function setColorMask(options) {
    var gl = this.renderer.gl;
    if (options == null && !this.doReset) return;
    if (!options) {
      if ((this.state & BIT_POS.colorMask) !== 0) {
        gl.colorMask(true, true, true, true);
        this.state &= ~BIT_POS.colorMask;
      }
    } else {
      gl.colorMask(!options[0], !options[1], !options[2], !options[3]);
      this.state |= BIT_POS.colorMask;
    }
  };

  StateManager.prototype.setDepthMask = function setDepthMask(value) {
    var gl = this.renderer.gl;
    if (value == null && !this.doReset) return;
    // If value is not 'false', it's enabled.
    var enabled = value !== false && value != null;
    if ((this.state & BIT_POS.depthMask) !== 0 === enabled) return;
    // Set the bit according to the bit set.
    if (enabled) {
      this.state |= BIT_POS.depthMask;
      gl.depthMask(false);
    } else {
      this.state &= ~BIT_POS.depthMask;
      gl.depthMask(true);
    }
  };

  StateManager.prototype.setCull = function setCull(options) {
    var gl = this.renderer.gl;
    this.setEnabled(gl.CULL_FACE, BIT_POS.cull, options);
    if (!options) return;
    if (typeof options === 'number') {
      // Assume CCW - it's basically a standard.
      gl.frontFace(gl.CCW);
      gl.cullFace(options);
    } else {
      gl.frontFace(options.front);
      gl.cullFace(options.face);
    }
  };

  StateManager.prototype.setDepth = function setDepth(options) {
    var gl = this.renderer.gl;
    this.setEnabled(gl.DEPTH_TEST, BIT_POS.depth, options);
    if (!options) return;
    if (typeof options === 'number') return gl.depthFunc(options);
    if (options.func != null) gl.depthFunc(options.func);
    if (options.mask != null) gl.depthMask(options.mask);
    if (options.range) gl.depthRange(options.range[0], options.range[1]);
  };

  StateManager.prototype.setStencil = function setStencil(options) {
    var gl = this.renderer.gl;
    this.setEnabled(gl.STENCIL_TEST, BIT_POS.stencil, options);
    if (!options) return;
    if (options.mask != null) {
      if (typeof options.mask === 'number') {
        gl.stencilMask(options.mask);
      } else {
        gl.stencilMaskSeparate(gl.FRONT, options.mask[0]);
        gl.stencilMaskSeparate(gl.BACK, options.mask[1]);
      }
    }
    if (options.func != null) {
      if (!Array.isArray(options.func[0])) {
        gl.stencilFunc(options.func[0], options.func[1], options.func[2]);
      } else {
        gl.stencilFuncSeparate(gl.FRONT, options.func[0][0], options.func[0][1], options.func[0][2]);
        gl.stencilFuncSeparate(gl.BACK, options.func[1][0], options.func[1][1], options.func[1][2]);
      }
    }
    if (options.op != null) {
      if (!Array.isArray(options.op[0])) {
        gl.stencilOp(options.op[0], options.op[1], options.op[2]);
      } else {
        gl.stencilOpSeparate(gl.FRONT, options.op[0][0], options.op[0][1], options.op[0][2]);
        gl.stencilOpSeparate(gl.BACK, options.op[1][0], options.op[1][1], options.op[1][2]);
      }
    }
  };

  StateManager.prototype.setViewport = function setViewport(value) {
    if (value == null && !this.doReset) return;
    // If value is not 'false', it's enabled.
    var enabled = value !== false && value != null;
    if (enabled) {
      var gl = this.renderer.gl;
      gl.viewport(value[0], value[1], value[2], value[3]);
      this.renderer.width = value[2];
      this.renderer.height = value[3];
    } else {
      // Revert to original viewport
      this.renderer.setViewport();
    }
  };

  StateManager.prototype.setScissor = function setScissor(options) {
    var gl = this.renderer.gl;
    this.setEnabled(gl.SCISSOR_TEST, BIT_POS.scissor, options);
    if (!options) return;
    gl.scissor(options[0], options[1], options[2], options[3]);
  };

  StateManager.prototype.setPolygonOffset = function setPolygonOffset(options) {
    var gl = this.renderer.gl;
    this.setEnabled(gl.POLYGON_OFFSET_FILL, BIT_POS.polygonOffset, options);
    if (!options) return;
    gl.polygonOffset(options[0], options[1]);
  };

  StateManager.prototype.clear = function clear(options) {
    var gl = this.renderer.gl;
    var flag = 0;
    if (options.clearColor) {
      var arr = (0, _parseUniform2.default)(gl, options.clearColor, gl.FLOAT_VEC4);
      // Whyyyyyyy
      gl.clearColor(arr[0], arr[1], arr[2], arr[3]);
      flag |= gl.COLOR_BUFFER_BIT;
    }
    if (options.clearDepth) {
      gl.clearDepth(options.clearDepth);
      flag |= gl.DEPTH_BUFFER_BIT;
    }
    if (options.clearStencil) {
      gl.clearStencil(options.clearStencil);
      flag |= gl.STENCIL_BUFFER_BIT;
    }
    if (flag !== 0) gl.clear(flag);
  };

  StateManager.prototype.setNode = function setNode(node) {
    var dirty = false;
    var options = {};
    (0, _traverseNode2.default)(this.currentNode, node, function (v) {
      dirty = true;
      for (var key in v.data.options) {
        var parent = v.parent;
        if (parent != null) {
          var value = parent.getOption(key);
          options[key] = value != null ? value : false;
        }
      }
    }, function (v) {
      dirty = true;
      for (var key in v.data.options) {
        options[key] = v.data.options[key];
      }
    });
    if (dirty) this.set(options, this.currentNode == null);
    this.currentNode = node;
  };

  StateManager.prototype.set = function set(options) {
    var reset = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

    this.doReset = reset;
    var gl = this.renderer.gl;
    this.setBlend(options.blend);
    this.setColorMask(options.colorMask);
    this.setDepthMask(options.depthMask);
    // this.clear(options);
    this.setCull(options.cull);
    this.setDepth(options.depth);
    this.setEnabled(gl.DITHER, BIT_POS.dither, options.dither);
    this.setStencil(options.stencil);
    this.setViewport(options.viewport);
    this.setScissor(options.scissor);
    this.setPolygonOffset(options.polygonOffset);
  };

  return StateManager;
}();

exports.default = StateManager;