'use strict';

exports.__esModule = true;

var _texture = require('./texture');

var _texture2 = _interopRequireDefault(_texture);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function isEmpty(source) {
  if (source == null) return true;
  if (Array.isArray(source)) return source.every(function (a) {
    return a == null;
  });
  return false;
}

var TextureManager = function () {
  function TextureManager(renderer) {
    _classCallCheck(this, TextureManager);

    this.renderer = renderer;
    this.textures = [];
    this.handler = null;

    // Active textures in LRU order
    this.activeTextures = [];

    var gl = renderer.gl;
    this.defaults = {
      target: renderer.gl.TEXTURE_2D,
      format: renderer.gl.RGB,
      type: renderer.gl.UNSIGNED_BYTE,
      params: {
        magFilter: gl.LINEAR,
        minFilter: gl.LINEAR_MIPMAP_LINEAR,
        wrapS: gl.CLAMP_TO_EDGE,
        wrapT: gl.CLAMP_TO_EDGE,
        mipmap: true,
        flipY: true
      }
    };
    this.emptyDefaults = {
      params: {
        mipmap: false,
        minFilter: gl.LINEAR
      }
    };
  }

  TextureManager.prototype.setDefault = function setDefault(options) {
    this.defaults = options;
  };

  TextureManager.prototype.create = function create(source, options) {
    var empty = isEmpty(source);
    var combinedOptions = Object.assign({}, this.defaults, empty && this.emptyDefaults, options, {
      source: source,
      params: Object.assign({}, this.defaults.params, empty && this.emptyDefaults.params, options && options.params)
    });
    var texture = new _texture2.default(this.renderer, combinedOptions);
    this.textures.push(texture);
    return texture;
  };

  TextureManager.prototype.use = function use(unit, texture, isFramebuffer) {
    if (unit == null || texture === false) return 0;
    // TODO This swaps texture unit even if that's unnecessary - However
    // anything but draw calls aren't expensive. So we'd just stick with it?
    // If the texture is already bound at right unit, don't do anything
    if (this.activeTextures[unit] === texture && texture.loaded && texture.valid && !texture.varyingSize) return unit;
    // Otherwise, bind the texture at specified unit.
    texture.use(unit, isFramebuffer);
    this.activeTextures[unit] = texture;
    return texture.unit;
  };

  TextureManager.prototype.reset = function reset() {
    if (this.renderer.anisotropic && this.defaults.params.maxAnisotropy == null) {
      this.defaults.params.maxAnisotropy = this.renderer.gl.getParameter(this.renderer.anisotropic.MAX_TEXTURE_MAX_ANISOTROPY_EXT);
    }
    // Unload all pre-loaded textures
    this.activeTextures = [];
    this.textures.forEach(function (texture) {
      return texture.dispose();
    });
  };

  return TextureManager;
}();

exports.default = TextureManager;