'use strict';

exports.__esModule = true;
exports.default = getAABB;

var _parseAttributes = require('./parseAttributes');

// Gets the AABB for selected geometry.
function getAABB(geom) {
  var attribName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'aPosition';

  var attribute = (0, _parseAttributes.parseAttribute)(geom.attributes[attribName]);
  // We don't have to check indices as long as all attributes are being used.
  var axis = attribute.axis,
      data = attribute.data;
  // Create AABB - it uses single Float32Array for both min / max values.

  var aabb = new Float32Array(axis * 2);
  for (var i = 0; i < axis; ++i) {
    aabb[i] = Infinity;
    aabb[axis + i] = -Infinity;
  }
  // Then test each vertex. :S
  for (var _i = 0; _i < data.length; _i += axis) {
    for (var j = 0; j < axis; ++j) {
      var current = data[_i + j];
      if (aabb[j] > current) aabb[j] = current;
      if (aabb[axis + j] < current) aabb[axis + j] = current;
    }
  }
  // All done!
  return aabb;
}