"use strict";

exports.__esModule = true;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// Represents each 'render pass' node. Used to memorize uniforms data, and
// avoid setting same uniforms / options twice.
var RenderNode = function () {
  function RenderNode(data, parent) {
    _classCallCheck(this, RenderNode);

    this.depth = 0;
    if (parent != null) this.depth = parent.depth + 1;
    this.parent = parent;

    if (this.depth === 0) this.root = this;else if (this.depth === 1) this.root = parent;else this.root = parent.root;

    this.data = data;
    this.framebuffer = data.framebuffer === undefined ? parent && parent.framebuffer : data.framebuffer;
    this.shader = data.shader || parent && parent.shader;
    this.geometry = data.geometry || parent && parent.geometry;
    this.shaderHandler = data.shaderHandler || parent && parent.shaderHandler;
    this.textureHandler = data.textureHandler || parent && parent.textureHandler;
  }

  RenderNode.prototype.get = function get(key) {
    if (this.data[key] !== undefined) return this.data[key];
    if (this.parent != null) return this.parent.get(key);
    return null;
  };

  RenderNode.prototype.getOption = function getOption(key) {
    var options = this.data.options;
    if (options != null && options[key] !== undefined) return options[key];
    if (this.parent != null) return this.parent.getOption(key);
    return null;
  };

  RenderNode.prototype.getUniform = function getUniform(key) {
    var uniforms = this.data.uniforms;
    if (uniforms != null && uniforms[key] !== undefined) return uniforms[key];
    if (this.parent != null) return this.parent.getUniform(key);
    return null;
  };

  return RenderNode;
}();

exports.default = RenderNode;